# -*- coding: utf-8 -*-
__date__ = '04/05 21:46'
__description__ = '''
'''

import urllib3
import sys
from pathlib import Path
from sgr_ansi import Chain
import sgr_ansi as echo
import click
from vto.core import copy_to_clipboard
from vto.dec import fmt_help

app_root = str(Path(__file__).parents[1])
sys.path.append(app_root)
urllib3.disable_warnings()

from btsniff.sites import SITES


@click.command(context_settings=dict(help_option_names=['-h', '--help'], terminal_width=200))
@click.option('--name', '-n', help=fmt_help('movie/actor name', '-n <name>'))
@click.option('--site', '-s', default='bbt', help=fmt_help('site name', '-s <site_name>'))
@click.option('--list_sites', '-l', is_flag=True, help=fmt_help('list supported sites', '-l'))
@click.option('--overwrite', '-w', is_flag=True, help=fmt_help('overwrite local cache', '-w'))
@click.option('--display_img', '-img', is_flag=True, help=fmt_help('display with image', '-img'))
def run(name, site, display_img, overwrite=False, list_sites=False):
    if list_sites:
        echo.BIy("Available sites:")
        for i, _site in enumerate(SITES.keys()):
            print(echo.BIm_(f'{_site:>8}'), echo.BIg_(f'[{SITES.get(_site).SiteURL.intro}]'))
        return
    chain = Chain()
    chain.BIUg(f'[SEARCH] site').BIUr(f'{site}').BIUg(f'for').BIUb(f'"{name}"').show()

    sniffer = SITES.get(site)
    link = sniffer.run(name, display_img, overwrite=overwrite)
    echo.IUb(link)
    copy_to_clipboard(link)


def debug_site(site, name, overwrite=False):
    """ local only """
    sniffer = SITES[site]
    sniffer.run(name, overwrite=overwrite)


if __name__ == '__main__':
    run()



__description__ = '''
url: bbt.tv
'''

import time
from dataclasses import dataclass
from pathlib import Path

from icraw import AsyncCrawler
import sgr_ansi as echo
from vto.core import num_choice
from vto import dec

app_root = Path(__file__).parents[1]

from btsniff.core import get_page_by_chrome, PageParser


@dataclass
class SiteURL:
    home: str = 'http://www.btbttv.cc/'
    # !WARN: UA Redirect ERROR
    search: str = 'http://www.btbttv.cc/index.php?s=vod-search'
    intro: str = f'bt电影天堂: {home}'


class BbtParser(PageParser):
    def _refine_torrent_name(self, info):
        return self.last_non_empty_info(info, index=0)


class Bbt(AsyncCrawler):
    def __init__(self, **kwargs):
        kwargs['site_init_url'] = SiteURL.home
        super().__init__(**kwargs)

    def search_name(self, name):
        cnt = self.bs4post(SiteURL.search, data={'wd': name}, ret='html')
        parser = BbtParser(raw_data=cnt)
        parser.do_parse()
        return parser.data['movies']

    def get_detail_page(self, url):
        url = url.replace('https://', 'http://')
        cnt = self.bs4get(url, is_json=False)
        parser = BbtParser(raw_data=cnt)
        parser.do_parse()
        return parser.data['torrents']

    @dec.prt(True)
    def get_final_link(self, url):
        url = url.replace('https://', 'http://')
        echo.BIg(f'>>> start to get thunder link:', end=' ')
        echo.BIU(url)
        dat = get_page_by_chrome(url)
        parser = BbtParser(raw_data=dat)
        parser.do_parse()
        return parser.data['thunder']


def run(name, display_img=False, overwrite=False):
    bbt = Bbt(overwrite=overwrite)

    dat = bbt.search_name(name)
    movies = [f'{m["movie"]["name"]}-{m["resolution"]}' for m in dat]
    movie_images = []
    if display_img:
        movie_images = [f"{m['image']}" for m in dat]
    c = num_choice(movies, img_list=movie_images, img_cache_dir=bbt.cache['site_media'])

    dat = dat[c]
    dat = bbt.get_detail_page(dat['movie']['link'])
    torrents = [f"{t['name']}" for t in dat]
    c = num_choice(torrents)

    link = bbt.get_final_link(dat[c]['link'])
    return link



from selenium import webdriver
from aibot import AiBot


def init_driver(headless=True):
    chrome_options = webdriver.ChromeOptions()
    # remove "chrome is being controlled by automated software"
    chrome_options.add_experimental_option('excludeSwitches', ['enable-automation'])
    # disable "save password" popup
    prefs = {
        "profile.password_manager_enabled": False,
        "credentials_enable_service": False,
        'profile.default_content_setting_values': {
            'images': 2,
            'notifications': 2,
            'javascript': 0,
            'geolocation': 0,
        }
    }
    chrome_options.add_experimental_option('prefs', prefs)
    if headless:
        chrome_options.add_argument('--headless')
    chrome_options.add_argument('--no-sandbox')
    chrome_options.add_argument('--disable-dev-shm-usage')
    driver = webdriver.Chrome(options=chrome_options)
    return driver


def get_page_by_chrome(url):
    driver = init_driver()
    driver.get(url)
    dat = driver.page_source
    driver.quit()
    return dat


def search_by_chrome(url, params, headless=True):
    driver = init_driver(headless)
    aib = AiBot(driver)
    aib.get(url)
    selector, txt = params
    aib.update_text(selector, txt + '\n')
    aib.bot.switch_to_window(1)
    dat = aib.driver.page_source
    aib.driver.quit()
    return dat



__description__ = '''url: https://www.btdx8.com'''

from dataclasses import dataclass
import sgr_ansi
from icraw import AsyncCrawler
from vto.core import num_choice

from btsniff.core import PageParser


@dataclass
class SiteURL:
    home: str = 'https://www.btdx8.com/'
    search: str = 'https://www.btdx8.com/?s={}'
    more: str = 'https://www.btdx8.com/down.php'
    intro: str = f'比特大雄: {home}'


class Btdx8Parser(PageParser):
    def _refine_torrent_name(self, info):
        return self.last_non_empty_info(info, index=0)


class Btdx8(AsyncCrawler):
    def __init__(self, **kwargs):
        kwargs['site_init_url'] = SiteURL.home
        super().__init__(**kwargs)

    def search_name(self, name):
        cnt = self.bs4get(SiteURL.search.format(name))
        parser = Btdx8Parser(raw_data=cnt)
        parser.do_parse()
        return parser.data['movies']

    def get_detail_page(self, url):
        cnt = self.bs4get(url)
        parser = Btdx8Parser(raw_data=cnt)
        parser.do_parse()
        dat = parser.data['torrents']
        return dat.get('direct', []) + dat.get('more', [])

    def load_more(self, url):
        cnt = self.bs4get(url)
        parser = Btdx8Parser(raw_data=cnt)
        parser.do_parse()
        print(parser.data)
        return parser.data


def run(name, display_img=False, overwrite=False):
    bt = Btdx8(overwrite=overwrite)

    dat = bt.search_name(name)
    movies = [f'{m["movie"]["name"]}' for m in dat]
    images = []
    if display_img:
        images = [f"{m['image']}" for m in dat]
    c = num_choice(movies, img_list=images, img_cache_dir=bt.cache['site_media'])

    dat = dat[c]
    details = bt.get_detail_page(dat['movie']['url'])
    torrents = [f"{t['name']}" for t in details]
    c = num_choice(torrents, depth=2)
    url = details[c]['url']
    if SiteURL.more not in url:
        return url
    else:
        sgr_ansi.BIr('    >>>url is not supported download, click following link, visit it manually!<<<\n')
        return url



__description__ = '''url: https://www.dy2018.com/'''

from dataclasses import dataclass
import sgr_ansi
from icraw import AsyncCrawler, ParseHeaderFromFile
from vto.core import num_choice
from vto.dec import prt
from ihelp import helper
from logzero import logger as zlog

from btsniff.core import PageParser, search_by_chrome


@dataclass
class SiteURL:
    home: str = 'https://www.dy2018.com/'
    search: str = 'https://www.dy2018.com/result'
    intro: str = f'电影天堂: {home}'


class DygodParser(PageParser):
    def _refine_torrent_name(self, info):
        return self.last_non_empty_info(info, index=0)

    def _refine_name(self, info):
        return self.last_non_empty_info(info, sep='=', index=-1)

    def _refine_url_href(self, info):
        if 'jianpian://pathtype=url' in info:
            return self.last_non_empty_info(info, sep='=', index=-1)
        return info


class Dygod(AsyncCrawler):
    def __init__(self, **kwargs):
        kwargs['site_init_url'] = SiteURL.home
        super().__init__(**kwargs)

    @prt(True)
    def search_name(self, name):
        pth, raw = self.load_cache(SiteURL.search, data={'keywords': name}, use_str=True)
        if not raw or self.overwrite:
            raw = search_by_chrome(SiteURL.home, ('input.formhue', name))
            helper.write_file(raw, pth)

        parser = DygodParser(raw_data=raw)
        parser.do_parse()
        return parser.data['movies']

    def get_detail_page(self, url: str) -> dict:
        """get movie candidates links

        Args:
            url (str): url

        Returns:
            dict: movies links
        """
        cnt = self.bs4get(url)
        parser = DygodParser(raw_data=cnt, encoding='gb2312')
        parser.do_parse()
        dat = parser.data['downloads']
        return dat


def run(name, display_img=False, overwrite=False):
    bt = Dygod(overwrite=overwrite)

    dat = bt.search_name(name)
    movies = [f'{m["name"]}' for m in dat]
    images = []
    if display_img:
        images = [f"{m['image']}" for m in dat]
    c = num_choice(movies, img_list=images, img_cache_dir=bt.cache['site_media'])

    dat = dat[c]
    details = bt.get_detail_page(dat['url'])
    candidates = [f"{t['name']}" for t in details]
    c = num_choice(candidates, depth=2)

    return details[c]['url']



from dataclasses import dataclass
from pathlib import Path
from iparse import IParser

app_root = Path(__file__).parents[1]
MOVIE_HOME_DIR = app_root / 'data'


@dataclass
class CNChar:
    COLON = '：'


class PageParser(IParser):
    def __init__(self, raw_data='', file_name='', is_test_mode=True, **kwargs):
        kwargs['startup_dir'] = kwargs.get('startup_dir', MOVIE_HOME_DIR)
        kwargs['log_level'] = 20
        if raw_data:
            kwargs['raw_data'] = raw_data
        super().__init__(file_name, is_test_mode=is_test_mode, **kwargs)

    def _refine_post_date(self, info):
        return self.last_non_empty_info(info, CNChar.COLON)



from setuptools import setup, find_packages
from btsniff import VERSION

setup(
    name='btsniff',
    version=VERSION,
    include_package_data=True,
    package_data={
        '': ['README.md'],
        'btsniff': ['data/*.yaml'],
    },
    packages=find_packages(),
    author='lihe',
    author_email='imanux@sina.com',
    url='https://github.com/coghost/btsniff',
    description='movie searcher',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    license='GPL',
    install_requires=[
        'icraw', 'iparse', 'lxml', 'click', 'urllib3', 'vto'
    ],
    entry_points={
        'console_scripts': ['btsniff=btsniff.app:run'],
    },
    project_urls={
        'Bug Reports': 'https://github.com/coghost/btsniff/issues',
        'Source': 'https://github.com/coghost/btsniff',
    },
    python_requires='>=3.7',
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords=['movie', 'movie searcher', 'douban']
)
